var frameModule = require("ui/frame");
var Color = require("color").Color;

var helpers = function() {};

helpers.color = function(hex, alpha = 1) {
  alpha = Math.min(Math.max(alpha, 0), 1);
  alpha = Math.round(255 * alpha);

  var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
  hex = hex.replace(shorthandRegex, function(m, r, g, b) {
    return r + r + g + g + b + b;
  });
  var regex = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
  var hexResult = (regex ? {
    r: parseInt(regex[1], 16),
    g: parseInt(regex[2], 16),
    b: parseInt(regex[3], 16)
  } : null);

  var value = new Color(alpha, hexResult.r, hexResult.g, hexResult.b);
  return value;
};

helpers.textShadow = function(view, props = {}) {
  if(props === true) props = {};
  var defaults = {
    color: "#000000",
    alpha: 1,
    x: 0, y: 0,
    radius: 2
  };
  for(var key in defaults) if(!props.hasOwnProperty(key)) props[key] = defaults[key];
  
  if(frameModule.topmost().ios) { // iOS
    view = view.ios;
    view.layer.masksToBounds = false;
    view.layer.shadowOpacity = props.alpha;
    view.layer.shadowRadius = props.radius;
    view.layer.shadowColor = new Color(props.color).ios.CGColor;
    view.layer.shadowOffset = CGSizeMake(props.x, props.y);
  }
};

helpers.nativeAnimation = function(view, props = {}, animations) {
  var defaults = {
    curve: "linear",
    duration: 250
  };
  for(var key in defaults) if(!props.hasOwnProperty(key)) props[key] = defaults[key];
  
  var curve = UIViewAnimationOptionCurveLinear,
      duration = props.duration / 1000;
  
  if(props.curve === "crossDissolve") curve = UIViewAnimationOptionTransitionCrossDissolve;
    else if(props.curve === "easeOut") curve = UIViewAnimationOptionCurveEaseOut;
    else if(props.curve === "easeInOut") curve = UIViewAnimationOptionCurveEaseInOut;
  
  return new Promise(function(resolve, reject) {
    UIView.transitionWithViewDurationOptionsAnimationsCompletion(view, duration, curve, animations, function() {
      resolve(true);
    });
  });
};

module.exports = helpers;